import math
from adap_gl import AdapGL as agl

def sqr( x ):
    return x * x

class C_t_tan_kernel:
    def __init__(self, df):
        self.df = df

    def v(self, theta):
        t = math.tan(theta)
        v = pow(1.0 / (1.0 + sqr(t) / self.df), (self.df + 1.0) / 2.0)
        return v / sqr(math.cos(theta))
                
class C_t_tan_density:
    def __init__(self, df):
        self.df = df
        f_kernel = C_t_tan_kernel(self.df)
        gl = agl()
        v = gl.integral( -math.pi/2.0, math.pi/2.0, f_kernel)
        self.c = 1.0 / v

    def v(self, theta):
        t = math.tan(theta)
        v = pow(1.0 / (1.0 + sqr(t) / self.df), (self.df + 1.0) / 2.0)
        return self.c * v / sqr(math.cos(theta))
        
class C_cum_t:
    def __init__(self, df):
        self.df = df
        self.t_tan = C_t_tan_density(self.df)
        self.gl = agl()

    def v( self, t ):
        t_cum = self.gl.integral(-math.pi/2.0, math.atan(t), self.t_tan)
        return t_cum

def p_value( t, df ):
    cum_t = C_cum_t(df)
    p  = cum_t.v(t)
    return p

